
;***************************************************************************
; 
; File Name		:'MORSE.asm"
; Title			:
; Date			:
; Version		:
; Support telephone	:765 287 1987  David B. VanHorn
; Support fax		:765 287 1989
; Support Email		:dvanhorn@cedar.net
; Support Snail		;1104 E 13th St, Muncie IN 47302
; Target MCU		:AT90S8515
; 
;***************************************************************************;
;	D E S C R I P T I O N
;
;An ASCII to MORSE converter, supporting all legal morse chars, plus 
;prosigns as defined in the ARRL handbook.
; 
;***************************************************************************;
;	M O D I F I C A T I O N   H I S T O R Y 
;
;
;       rev.      date    who   why
;	----	--------  ---	------------------------------------------
;	0.01	98.10.19  dvh	Creation
;
;
;********************************************************************

;*************************************************************
;Beep Engine
;*************************************************************
;
;The beep engine is fed a byte value parsed as tone/duration
;
;
;Beep-Bit-Map
;
;	7	Beep Method	
;	6
;
;	(00)	Audio tone	Not supported
;	(01)	CW		Bit on = beep, bit off = silence
;	(10)	FSK		Not supported
;	(11)	???
;
;	5	Unused, mask off.	
;	4	1 = Beep 0 = Silent
;
;	3	Tone or silence duration
;	2	
;	1	1-8 "dots" (N+1)
;	0
;
;
;Beepit takes a beep-byte and outputs it as appropriate.
;Tone burst, Carrier enable, or FSK, for some duration.
;Exits with tone off, freq normal, or carrier off as approp.
;
;At this time, only CW is implemented, so we don't eve worry about the other modes.
;Later, put some code here to parse off the modes, and vector to appropriate handlers.
;
Beepit:	
	lds	TEMP,BEEPFLAG	;
	andi	TEMP,$C0	;Only the method remains
	cpi	TEMP,$00	;
	breq	Beep_Tone	;
	cpi	TEMP,$40	;
	breq	Beep_CW		;
	cpi	TEMP,$80	;
	breq	BEEP_FSK	;

BEEP_Tone:	;Not implemented
	ret

BEEP_FSK:	;Not implemented
	ret	

BEEP_CW:
	lds	TEMP,BEEPFLAG	;movfw	BEEPFLAG	;
	andi	TEMP,$10	;andlw	b'00010000'	;To beep, or not to beep.
	breq	BEEP_MRK_CW	;bz	BEEP_MRK_CW	;

BEEP_SPC_CW:

	sbi	PORTA,7		;BSF	PORTB,2		;Set carrier on
	rjmp	BEEP_Delay	;goto	BEEP_DELAY	;Wait our appointed bit-time

BEEP_MRK_CW:
	cbi	PORTA,7		;BCF	PORTB,2		;Set carrier off
	rjmp	BEEP_Delay	;goto	BEEP_DELAY	;
;
;This is a time-waster that takes the same input but just sits here.
;
BEEP_DELAY:
	lds	TEMP,BEEPFLAG	;movfw	BEEPFLAG	;Get the time value
	andi	TEMP,$0F	;andlw	b'00001111'	;Trim other junk
				;movwf	SUBSEC		;Store it.
	ROL	TEMP		;rlf	SUBSEC,1	;
	ROL	TEMP		;rlf	SUBSEC,1	;Brings the low CW speed to abt 3WPM
	sts	SUBSEC,TEMP	;

BEEP_DA:
	lds	TEMP,SPEED	;movfw	SPEED		;About 25mS per dit
	sts	SUBSECA,TEMP	;movwf	SUBSECA		;

BEEP_DB:
	lds	TEMP,SUBSECA	;movfw	SUBSECA		;Check if it's time to quit
	and	TEMP,TEMP	;
	breq	BEEP_DB		;bnz	BEEP_DB		;
	dec	TEMP		;decf	SUBSEC,1	;
	breq	BEEP_DA		;BNZ	BEEP_DA		;If zero, then just quit

BEEP_DONE:
	ret
;
;
;
;*************************************************************
;ASCII to Morse conversion
;*************************************************************
;
;I initially wrote this on a PIC, and thought it would be amusing to do a line-by line
;comparison between the pic and the AVR. The PIC was an F84, which has 64 registers and 
;no SRAM as such. Given the way lookup tables on the PIC work, I would have coded it 
;differently in the AVR, if I was starting from scratch.
;
;Convert a char of ASCII to corresponding morse code.
;Non-supported chars are ignored.
;Called with char in MORSE
;
;SRAM: MORSE DBL, DBH, DBT, BEEPFLAG, SPEED, SUBSEC, SUBSECA
;It costs an additional 2 instruction penalty to retrieve and to store these,
;but I decided not to eat up all the reg space with them.
;
;
ASC2MORSE:
	lds	TEMP,MORSE	;movlw	h'20'		;Make the offset
	subi	TEMP,$20	;subwf	MORSE,1		;Space is now zero
	breq	MORSESPC	;bz	MORSESPC	;Trap out the space char, send as six dots of silence.
	subi	TEMP,2		;movlw	2		;Space and next are not encoded.
	sts	MORSE,TEMP	;subwf	MORSE,1		;
	rjmp	A2M		;goto	A2M		;return legal values to the beep engine.
	
;Special handler for space. Not encoded in table
MORSESPC:
	lds	TEMP,BEEPFLAG	;movlw	b'11000000'	;
	andi	TEMP,$C0	;andwf	BEEPFLAG,1	;Clean out junk	
				;movlw	b'00000110'	;Start beep engine with pre-char delay
	ori	TEMP,$06	;iorwf	BEEPFLAG,1	;Send 6 dots of silence (Inter-char delay)	
	sts	BEEPFLAG,TEMP	;
	rcall	Beepit		;call	BEEPIT		;because the prev char will have sent a single
	rjmp	Morse_Done	;goto	MORSE_DONE	;silent dot, the total is 7

;All chars except space come here
A2M:	
	lds	TEMP,BEEPFLAG	;movlw	b'11000000'	;
	andi	TEMP,$C0	;andwf	BEEPFLAG,1	;Clean out junk	
				;movlw	b'00000010'	;Start beep engine with pre-char delay
	ori	TEMP,$02	;iorwf	BEEPFLAG,1	;Send 2 dots of silence, the prev char sends 1	
	sts	BEEPFLAG,TEMP	;
	rcall	Beepit		;call	BEEPIT		;
	clr	TEMP		;
	sts	DBL,TEMP	;clrf	DBL		;Clear out the di-bit indexes.
	sts	DBH,TEMP	;clrf	DBH		;
	rjmp	A2MAA		;goto	A2MAA		;

	;Put target char in temp
	;search through morse table till match.
A2MA:
	;Point at the next di-bit
	clc			;bcf	STATUS,0	;Reset carry bit
	lds	TEMP,DBL	;
	inc	TEMP		;incf	DBL,1		;Point at the next di-bit
	sts	DBL,TEMP	;
	brne	A2MAA		;BNZ	A2MAA		;
	lds	TEMP,DBH	;
	inc	TEMP		;
	sts	DBH,TEMP	;incf	DBH,1		;

A2MAA:
	;If not, then scan di-bits till we hit a zero
				;Complicated move saves 2 more bytes.
	lds	TEMP,DBH	;movfw	DBH		;	
				;movwf	TEMP		;
	ror	TEMP		;rrf	TEMP,1		;thru carry
	ror	TEMP		;rrf	TEMP,1		;and into position
	ror	TEMP		;rrf	TEMP,1		;Temp now has low 2 bits from DBH in XX------
				;movfw	TEMP		;in W now
	andi	TEMP,$C0	;andlw	b'11000000'	;
	sts	DBT,TEMP	;movwf	DBT		;Put it in the Di-Bit Temp

	lds	TEMP,DBL	;movfw	DBL		;
				;movwf	TEMP		;
	ror	TEMP		;rrf	TEMP,1		;/2
	ror	TEMP		;rrf	TEMP,1		;/4
				;movfw	TEMP		;
	andi	TEMP,$3F	;andlw	b'00111111'	;
				;iorwf	DBT,0		;W now has the address pointer (YAY!)

	;rcall	MORSETRAN	;call	MORSETRAN	;Returns the current byte in W
	;MORSETRAN picks the bytes out of the table, puts them into temp

	sts	DBT,TEMP	;movwf	DBT		;Store the set of di-bits here

	;Use DBL ------XX to extract the proper di-bit
	;movlw	3		;Set the !#%!(_#% page "feature"
	;movwf	PCLATH		;
	lds	TEMP,DBL	;movfw	DBL		;
	andi	TEMP,$03	;andlw	b'00000011'	;Strip all but what I need
	;Now this tells me which di-bit I'm looking at.
	;addwf	PCL,1		;
	;goto	MBITSA		;
	;goto	MBITSB		;
	;goto	MBITSC		;
	;goto	MBITSD		;

MBITSA:
	lds	TEMP,DBT	;movfw	DBT		;Get it back
	andi	TEMP,$C0	;andlw	b'11000000'	;
	breq	MBITS_Zero	;bz	MBITS_ZERO	;
	rjmp	MBITS_NZ	;goto	MBITS_NZ	;

MBITSB:
	lds	TEMP,DBT	;movfw	DBT		;Get it back
	andi	TEMP,$30	;andlw	b'00110000'	;
	breq	MBITS_ZERO	;bz	MBITS_ZERO	;
	rjmp	MBITS_NZ	;goto	MBITS_NZ	;

MBITSC:
	lds	TEMP,DBT	;movfw	DBT		;Get it back
	andi	TEMP,$0C	;andlw	b'00001100'	;
	breq	MBITS_ZERO	;bz	MBITS_ZERO	;
	rjmp	MBITS_NZ	;goto	MBITS_NZ	;

MBITSD:
	lds	TEMP,DBT	;movfw	DBT		;Get it back
	andi	TEMP,$03	;andlw	b'00000011'	;
	breq	MBITS_ZERO	;bz	MBITS_ZERO	;
	rjmp	MBITS_NZ	;goto	MBITS_NZ	;

MBITS_ZERO:
	lds	TEMP,MORSE	;movfw	MORSE		;
	and	TEMP,TEMP	;
	breq	MORSE_Done	;BZ	MORSE_DONE	;If the dibit's a zero and morse is zero,
				;then we're all done!

	dec	TEMP		;
	sts	MORSE,TEMP	;decf	MORSE,1		;Got a zero, so we've moved a char
	rjmp	A2MA		;goto	A2MA		;

;
;Here, we have non-zero dibits in DBT, but we only want to send them
;if MORSE is zero, AND until we hit non-zero dibits (Taken care of above)
;
MBITS_NZ:
	sts	DBT,TEMP	;movwf	DBT		;Save it
	lds	TEMP,MORSE	;movfw	MORSE		;
	and	TEMP,TEMP	;
	breq	A2MA		;BNZ	A2MA		;non-zero, just ignore
	;Here if MORSE is zero

MBITS_RE:
	lds	TEMP,DBT	;movfw	DBT		;
	andi	TEMP,$03	;andlw	b'00000011'	;Anything here?
	brne	MBITS_Send	;bnz	MBITS_SEND	;Yes, send it
	lds	TEMP,DBT
	ror	TEMP		;rrf	DBT,1		;else Shift the dibits right 
	ror	TEMP		;rrf	DBT,1		;
	sts	DBT,TEMP	;
	rjmp	MBITS_RE	;goto	MBITS_RE	;and try again

MBITS_SEND:
	;Send a beep, either dot or dash
	lds	TEMP,BEEPFLAG	;movlw	b'11000000'	;
	andi	TEMP,$C0	;andwf	BEEPFLAG,1	;Clean out old junk (xcpt mode)
				;movlw	b'00010000'	;
	ori	TEMP,$10	;iorwf	BEEPFLAG,1	;send a tone
	lds	TEMP2,DBT	;movfw	DBT		;
	or	TEMP,TEMP2	;iorwf	BEEPFLAG,1	;Set the duration
	sts	DBT,TEMP	;
	rcall	Beepit		;call	BEEPIT		;

	;Always follow with a dot of silence
	lds	TEMP,BEEPFLAG	;movlw	b'11000000'	;
	andi	TEMP,$C0	;andwf	BEEPFLAG,1	;Clean out old junk (xcpt mode)

				;movlw	b'00000001'	;send a dot of silence
	ori	TEMP,$01	;iorwf	BEEPFLAG,1	;
	sts	BEEPFLAG,TEMP	;
	rcall	BEEPIT		;call	BEEPIT		;
	rjmp	A2MA		;goto	A2MA		;

MORSE_DONE:
	ret
;
